@@ -19,9 +19,9 @@
 #define _GNU_SOURCE
 #define _FILE_OFFSET_BITS 64
 
-#include <configure.h>
+#include "configure.h"
 #include <assert.h>
-#include <libdevmapper.h>
+#include "libdevmapper.h"
 
 #include "lvm-types.h"
 #include "lvm-logging.h"
@@ -114,7 +114,7 @@ struct arg {
 	uint64_t ui64_value;
 	sign_t sign;
 	percent_t percent;
-	void *ptr;
+/*	void *ptr; // Currently not used. */
 };
 
 #define CACHE_VGMETADATA 0x00000001
